import * as React from 'react'
import {CSSProperties, ReactNode, useState} from 'react'
import { darken, Icon, IconButton, Theme } from '@mui/material';
import createStyles from '@mui/styles/createStyles';
import makeStyles from '@mui/styles/makeStyles';
import classNames from 'classnames'
import {usePersistentState} from 'react-persistent-state/build'
import {colorError, colorInfo, colorSuccess, colorWarning} from '../core/style/color'

const height = (dense?: boolean) => dense ? 44 : 52

const useStyles = makeStyles<Theme, AlertProps>((t: Theme) => createStyles({
  root: {
    paddingLeft: t.spacing(2),
    paddingRight: t.spacing(2),
    [t.breakpoints.up('sm')]: {
      paddingLeft: t.spacing(3),
      paddingRight: t.spacing(3),
    },
    transition: t.transitions.create('all'),
    // @ts-ignore
    minHeight: props => height(props.dense),
    display: 'flex',
    alignItems: 'center',
    overflow: 'hidden',
    borderRadius: 4,
    // @ts-ignore
    paddingLeft: props => props.dense ? t.spacing(1) : t.spacing(2),
  },
  i: {
    marginRight: t.spacing(1),
    height: props => `${height(props.dense)} !important`,
    display: 'flex',
    alignItems: 'center',
    alignSelf: 'flex-start',
  },
  body: {
    flex: 1,
    paddingTop: props => t.spacing(props.dense ? 1 : 2),
    paddingBottom: props => t.spacing(props.dense ? 1 : 2),
  },
  _success: {
    background: 'rgba(50, 255, 150, .08)',//'#e1ffe1',
    color: darken(colorSuccess, .1),
  },
  _info: {
    background: 'rgba(50, 200, 255, .08)', //'#e1f5fe',
    color: darken(colorInfo, .1),
  },
  _error: {
    background: 'rgba(255, 0, 0, .08)',//'#ffdede',
    color: darken(colorError, .1),
  },
  _warning: {
    background: 'rgba(255, 128, 0, .08)',
    color: darken(colorWarning, .1),
  },
  _hidden: {
    minHeight: '0 !important',
    height: '0 !important',
    opacity: '0 !important',
    margin: '0 !important',
  },
  _gutterBottom: {
    marginBottom: t.spacing(1),
  },
  action: {
    textAlign: 'right',
    margin: `${t.spacing(1)} ${t.spacing(1 / 2)} ${t.spacing(1 / 2)} 0`,
    marginRight: -t.spacing(2),
  }
}))

interface AlertProps {
  id?: string
  style?: CSSProperties
  type: 'info' | 'error' | 'warning' | 'success'
  icon?: string
  hidden?: boolean
  deletable?: boolean
  persistentDelete?: boolean
  action?: ReactNode
  className?: string
  children: ReactNode
  dense?: boolean
  gutterBottom?: boolean
}

export const Alert = ({id, style, type, dense, children, hidden, icon, action, deletable, persistentDelete, className, gutterBottom}: AlertProps) => {
  const classes = useStyles({type, dense, children, icon, action, deletable, persistentDelete, className})
  const [isPersistentVisible, setPersistentIsVisible] = usePersistentState<boolean>(true, id || 'alert')
  const [isVisible, setIsVisible] = useState<boolean>(true)

  const getIconFromType = () => {
    switch (type) {
      case 'info':
        return 'info'
      case 'error':
        return 'error'
      case 'warning':
        return 'warning'
      case 'success':
        return 'check_circle'
      default:
        return 'info'
    }
  }

  return (
    <div
      id={id}
      style={style}
      className={classNames(
        className,
        classes.root,
        classes['_' + type],
        (hidden || !isVisible || (persistentDelete && !isPersistentVisible)) && classes._hidden,
        gutterBottom && classes._gutterBottom
      )}
    >
      <Icon className={classes.i}>{icon ? icon : getIconFromType()}</Icon>
      <div className={classes.body}>
        {children}
      </div>
      <div className={classes.action}>
        {action}
        {deletable &&
        <IconButton
          onClick={() => {
            setIsVisible(false)
            if (persistentDelete)
              setPersistentIsVisible(false)
          }}
          size="large">
          <Icon>clear</Icon>
        </IconButton>
        }
      </div>
    </div>
  );
}
